<?php
/**
 * WholesaleX Conversation Admin Page
 *
 * @package WHOLESALEX_PRO
 * @since 1.0.0
 */

namespace WHOLESALEX_PRO;

use WP_Query;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX Conversation
 */
class Conversation {


	/**
	 * Constructor
	 */
	public function __construct() {
		$this->conversation_post_type_callback();
		add_action( 'admin_menu', array( $this, 'conversation_menu_callback' ) );
		add_action( 'rest_api_init', array( $this, 'register_conversation_restapi' ) );

		add_action( 'add_meta_boxes', array( $this, 'reply_metabox_callback' ) );
		add_action( 'save_post', array( $this, 'reply_metabox_save_callback' ) );
		add_filter( 'gettext', array( $this, 'change_publish_button' ), 10, 2 );
		add_action( 'wp_ajax_wsx_users', array( $this, 'user_search_callback' ) );
		add_filter( 'wholesalex_setting_fields', array( $this, 'conversation_settings_fields' ) );
		add_action( 'wp_ajax_conversation_status_change', array( $this, 'conversation_status_change' ) );

		/**
		 * Sort Coversations by last reply time.
		 *
		 * @since 1.0.2
		 */
		add_filter( 'pre_get_posts', array( $this, 'sort_by_last_reply_time' ) );

		/**
		 * Hide Quote Request When Request a Quote Addon Disabled.
		 *
		 * @since 1.0.5
		 */
		add_filter( 'pre_get_posts', array( $this, 'hide_quotes_when_disable_quote' ) );

		add_action( 'in_admin_header', array( $this, 'add_conversation_header' ) );

		add_filter( 'woocommerce_email_classes', array( $this, 'wholesalex_add_email_classes') );
	}

	/**
	 * Add Conversation Email In Woo Settings
	 *
	 * @param [array] $email_classes
	 * @return array
	 */
	public function wholesalex_add_email_classes( $email_classes ) {
		$email_classes['WholesaleX_New_Conversation_Email'] 		= include WHOLESALEX_PRO_PATH .'/addons/conversation/emails/class-wholesalex-conversation-email.php';
		$email_classes['WholesaleX_New_Conversation_Reply_Email'] 	= include WHOLESALEX_PRO_PATH .'/addons/conversation/emails/class-wholesalex-conversation-reply-email.php';
		return $email_classes;
	}
	/**
	 * Add Header on Edit Conversation Page
	 */
	public function add_conversation_header() {
        if(isset($_GET['post']) && 'wsx_conversation' == sanitize_key(get_post_type( $_GET['post'] ))) { //phpcs:ignore
			?>
				<div id="wholesalex_coversation_header"></div>
			<?php
			wp_enqueue_script( 'whx_conversation' );
			wp_enqueue_script( 'wholesalex_node_vendors' );
			wp_enqueue_script( 'wholesalex_components' );
		}
	}

	/**
	 * Hide Quote Request While Disable Quote Addon
	 *
	 * @param object $wp_query WP Query.
	 * @return object
	 * @since 1.0.5
	 */
	public function hide_quotes_when_disable_quote( $wp_query ) {
		global $pagenow;
		if ( current_user_can( 'edit_posts' ) && is_admin() && 'edit.php' === $pagenow ) {
			$post_type = $wp_query->query['post_type'];
			if ( 'wsx_conversation' === $post_type ) {
				$wp_query->set( 'meta_key', '__conversation_type' );
				$wp_query->set( 'meta_value', ( 'yes' !== wholesalex()->get_setting( 'wsx_addon_raq' ) ) ? 'quote' : '' );
				$wp_query->set( 'meta_compare', '!=' );
			}
		}

		return $wp_query;
	}

	/**
	 * Conversation Status Change
	 *
	 * @return void
	 * @since 1.0.5
	 */
	public function conversation_status_change() {
		if ( isset( $_POST['nonce'], $_POST['conversationID'] ) && wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'view_conversation_' . sanitize_text_field( wp_unslash( $_POST['conversationID'] ) ) ) ) {
			$id = isset( $_POST['conversationID'] ) ? sanitize_text_field( wp_unslash( $_POST['conversationID'] ) ) : '';
			if ( '' !== $id ) {
				update_post_meta( $id, '__conversation_status', 'open' );
				wp_send_json_success( __('Conversation Status Changed to Open','wholesalex-pro') );
			}
			wp_send_json_error( 'Failed' );
		}
	}
	/**
	 * Conversation User Search
	 *
	 * @return mixed
	 */
	public function user_search_callback() {
		if ( ! ( isset( $_POST['wpnonce'] ) && wp_verify_nonce( sanitize_key( $_POST['wpnonce'] ), 'wholesalex-registration' ) ) ) {
			return;
		}

		global $wpdb;
		$term         = '%' . $wpdb->esc_like( isset( $_POST['term'] ) ? sanitize_text_field( wp_unslash( $_POST['term'] ) ) : '' ) . '%';
		$post_results = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT ID, display_name FROM $wpdb->users  WHERE user_login LIKE %s OR ID LIKE %s OR user_nicename LIKE %s OR user_email LIKE %s OR display_name LIKE %s LIMIT 10",
				$term,
				$term,
				$term,
				$term,
				$term
			)
		);
		$data         = '';
		if ( ! empty( $post_results ) ) {
			foreach ( $post_results as $key => $val ) {
				$data .= '<li data-id="' . $val->ID . '">[ID: ' . $val->ID . '] ' . $val->display_name . '</li>';
			}
		}
		wp_send_json_success( $data );
	}

	/**
	 * Change Conversation Publish Button Text
	 *
	 * @param string $translation Translated Text.
	 * @param string $text Original Text.
	 * @return string updated text.
	 * @since 1.0.0
	 */
	public function change_publish_button( $translation, $text ) {
		if ( 'wsx_conversation' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return __( 'Create Conversation', 'wholesalex-pro' );
			} elseif ( 'Update' === $text ) {
				return __( 'Reply / Update', 'wholesalex-pro' );
			}
		}
		return $translation;
	}

	/**
	 * Conversation Post Type Callback: It register Conversation Post Type.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function conversation_post_type_callback() {
		$menu_title = apply_filters( 'wholesalex_addon_conversation_title', _x( 'Conversations', 'Post type general name', 'wholesalex-pro' ) );
		$labels     = array(
			'name'               => $menu_title,
			'singular_name'      => _x( 'Conversation', 'Post type singular name', 'wholesalex-pro' ),
			'menu_name'          => _x( 'Conversations', 'Admin Menu text', 'wholesalex-pro' ),
			'name_admin_bar'     => _x( 'Conversation', 'Add New on Toolbar', 'wholesalex-pro' ),
			'add_new'            => __( 'Add New', 'wholesalex-pro' ),
			'add_new_item'       => __( 'Add New Conversation', 'wholesalex-pro' ),
			'new_item'           => __( 'New Conversation', 'wholesalex-pro' ),
			'edit_item'          => __( 'Edit Conversation', 'wholesalex-pro' ),
			'view_item'          => __( 'View Conversation', 'wholesalex-pro' ),
			'all_items'          => __( 'All Conversations', 'wholesalex-pro' ),
			'search_items'       => __( 'Search Conversations', 'wholesalex-pro' ),
			'parent_item_colon'  => __( 'Parent Conversations:', 'wholesalex-pro' ),
			'not_found'          => __( 'No Conversations found.', 'wholesalex-pro' ),
			'not_found_in_trash' => __( 'No Conversations found in Trash.', 'wholesalex-pro' ),
		);
		$args       = array(
			'labels'             => $labels,
			'description'        => __( 'Conversation custom post type.', 'wholesalex-pro' ),
			'public'             => false,
			'publicly_queryable' => false,
			'show_ui'            => true,
			'show_in_menu'       => false,
			'query_var'          => true,
			'rewrite'            => array( 'slug' => 'wsx_conversation' ),
			'capability_type'    => 'post',
			'has_archive'        => true,
			'hierarchical'       => true,
			'menu_position'      => 20,
			'supports'           => array( 'title' ),
			'show_in_rest'       => true,
		);
		register_post_type( 'wsx_conversation', $args );
	}
	/**
	 * Conversation Menu Callback
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function conversation_menu_callback() {
		$__pending_conversation = $this->get_open_conversation_count();
		$menu_title             = apply_filters( 'wholesalex_addon_conversation_title', __( 'Conversations', 'wholesalex-pro' ) );
		$menu_endpoint          = apply_filters( 'wholesalex_addon_conversation_endpoint', 'wholesalex-conversation' );

		$menu_slug = apply_filters( 'wholesalex_plugin_menu_slug', 'wholesalex-overview' );

		add_submenu_page(
			$menu_slug,
			$menu_title,
			/* translators: %s: number of pending conversation */
			$__pending_conversation ? sprintf( __( '%1$s <span class="menu-counter"><span>%2$d</span></span>', 'wholesalex-pro' ), $menu_title, $__pending_conversation ) : $menu_title,
			apply_filters( 'wholesalex_capability_access', 'manage_options' ),
			$menu_endpoint,
			array( $this, 'conversation_page_content' ),
			4
		);
	}

	public function register_conversation_restapi() {
		register_rest_route(
			'wholesalex/v1',
			'/conversation/',
			array(
				array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'conversation_restapi_callback' ),
					'permission_callback' => array( $this, 'restapi_permission_callback' ),
					'args'                => array(),
				),
			)
		);
	}

	public function restapi_permission_callback() {
		$capability_access = apply_filters( 'wholesalex_capability_access', 'manage_options' );
		$status = apply_filters( 'wholesalex_conversation_restapi_permission_callback', current_user_can( $capability_access ) );
		return $status;
	}

	public function conversation_restapi_callback( $server ) {
		$post = $server->get_params();
		if ( ! ( isset( $post['nonce'] ) && wp_verify_nonce( $post['nonce'], 'wholesalex-registration' ) ) ) {
			return;
		}

		$type = isset( $post['type'] ) ? sanitize_text_field( $post['type'] ) : '';

		$response = array(
			'status' => false,
			'data'   => array(),
		);

		switch ( $type ) {
			case 'get':
				$page              = isset( $post['page'] ) ? sanitize_text_field( $post['page'] ) : 1;
				$status            = isset( $post['status'] ) ? sanitize_text_field( $post['status'] ) : '';
				$search_query      = isset( $post['search'] ) ? sanitize_text_field( $post['search'] ) : '';
				$conversation_type = isset( $post['conversation_type'] ) ? sanitize_text_field( $post['conversation_type'] ) : '';
				$post_status       = isset( $post['post_status'] ) && ! empty( $post['post_status'] ) ? sanitize_text_field( $post['post_status'] ) : 'publish';

				$response['status'] = true;
				$isFrontend         = isset( $post['isFrontend'] ) ? true : false;
				$response['data']   = $this->get_conversations( 10, $page, $status, $search_query, $conversation_type, $post_status, $isFrontend );
				break;
			case 'update_status':
				$action = isset( $post['conversation_action'] ) ? sanitize_text_field( $post['conversation_action'] ) : '';
				$id     = isset( $post['id'] ) ? sanitize_text_field( $post['id'] ) : '';

				if ( empty( $action ) ) {
					$response['status'] = false;
					$response['data']   = 'Invalid Action!';
				}
				if ( empty( $id ) ) {
					$response['status'] = false;
					$response['data']   = 'Invalid User Id!';
				}

				if ( ! empty( $action ) && ! empty( $id ) ) {
					$this->handle_conversation_action( $action, $id );

					$response['status'] = true;
					$response['data']   = 'delete' === $action ? __('Successfully Deleted','wholesalex-pro') : __('Successfully Updated','wholesalex-pro');

				}
				break;

			case 'bulk_action':
				$action = isset( $post['conversation_action'] ) ? sanitize_text_field( $post['conversation_action'] ) : '';
				$ids    = isset( $post['ids'] ) ? wholesalex()->sanitize( $post['ids'] ) : array();
				if ( empty( $action ) ) {
					$response['status'] = false;
					$response['data']   = __('Invalid Action!','wholesalex-pro');
				}
				if ( empty( $ids ) || ! is_array( $ids ) ) {
					$response['status'] = false;
					$response['data']   = __('Invalid User Ids!','wholesalex-pro');
				}

				if ( ! empty( $action ) && ! empty( $ids ) && is_array( $ids ) ) {
					$this->bulk_actions( $action, $ids );

					$response['status'] = true;
					$response['data']   = 'delete' === $action ? __('Successfully Deleted','wholesalex-pro') : __('Successfully Updated','wholesalex-pro');

				}

				break;
			default:
				// code...
				break;
		}

		wp_send_json( $response );
	}

	  // Bulk Actions
	public function bulk_actions( $action, $ids ) {
		switch ( $action ) {
			case 'new':
			case 'open':
			case 'resolve':
			case 'delete':
			case 'move_to_trash':
			case 'untrash':
				if ( is_array( $ids ) ) {
					foreach ( $ids as $id ) {
						$this->handle_conversation_action( $action, $id );
					}
				}
				break;
			default:
				// code...
				break;
		}
	}

	public function handle_conversation_action( $action, $conversation_id ) {

		switch ( $action ) {
			case 'new':
			case 'open':
			case 'resolved':
				update_post_meta( $conversation_id, '__conversation_status', $action );
				break;
			case 'delete':
				wp_delete_post( $conversation_id, true );
				break;
			case 'move_to_trash':
				wp_trash_post( $conversation_id );
				break;
			case 'untrash':
				wp_untrash_post( $conversation_id );
				break;

			default:
				// code...
				break;
		}
	}


	public function get_conversations( $per_page = -1, $page = 1, $status = '', $search_query = '', $type = '', $post_status = 'publish', $is_frontend=false ) {
		$args = array(
			'posts_per_page' => $per_page,
			'post_type'      => 'wsx_conversation',
			'orderby'        => 'modified',
			'order'          => 'DESC',
			'paged'          => $page,
			's'              => $search_query,
			'post_status'    => $post_status,
			'count_total'    => true,
		);
		if ( '' != $status ) {
			$status_query = array(
				array(
					'key'     => '__conversation_status',
					'value'   => $status,
					'compare' => '=',
				),
			);
			if ( isset( $args['meta_query'] ) && is_array( $args['meta_query'] ) ) {
				$args['meta_query'][] = $status_query;
			} else {
				$args['meta_query'] = $status_query;
			}
		}
		if ( '' != $type ) {
			$type_query = array(
				array(
					'key'     => '__conversation_type',
					'value'   => $type,
					'compare' => '=',
				),
			);
			if ( isset( $args['meta_query'] ) && is_array( $args['meta_query'] ) ) {
				$args['meta_query'][] = $type_query;
			} else {
				$args['meta_query'] = $type_query;
			}
		}
		$args = apply_filters( 'wholesalex_get_conversations_args', $args, $is_frontend );
		if ( isset( $args['meta_query'] ) && count( $args['meta_query'] ) > 1 ) {
			$args['relation'] = 'AND';
		}
		$conversation_search = new WP_Query( $args );
		$total_conversations = $conversation_search->found_posts;
		$conversations       = $conversation_search->posts;

		$column_value = array();
		foreach ( $conversations as $key => $conversation ) {
			$column_data          = array();
			$conversation_id      = $column_data['ID'] = $conversation->ID;
			$column_data['title'] = $conversation->post_title;
			$column_data['user']  = $conversation->post_title;

			$conversation_type = get_post_meta( $conversation_id, '__conversation_type', true );
			$is_quote          = ( 'quote' === $conversation_type );
			$username          = '';
			$email             = '';
			$author_id         = get_post_field( 'post_author', $conversation_id );

			$user = get_userdata( $author_id );
			if ( ! $author_id ) {
				$column_data['user'] = __( 'Guest', 'wholesalex-pro' );
			} else {
				$column_data['user']  = $user ? $user->display_name : __( 'Deleted User', 'wholesalex-pro' );
				$column_data['email'] = $user ? $user->user_email : __( 'Email Not Found!', 'wholesalex-pro' );
			}

			if ( $is_quote ) {
				$is_guest   = $author_id === 0;
				$quote_data = get_post_meta( $conversation_id, '__wholesalex_quote_data', true );
				$quote_data = json_decode( $quote_data, true );
				if ( $is_guest ) {
					$column_data['user'] = isset( $quote_data['name'] ) ? $quote_data['name'] . '(Guest)' : 'Guest';
				} else {
					$column_data['user'] = isset( $quote_data['name'] ) ? $quote_data['name'] : $column_data['user'];
				}
				if ( $quote_data['user_email'] && is_email( $quote_data['user_email'] ) ) {
					$email = $quote_data['user_email'];
				} elseif ( $user ) {
					$email = $user->user_email;
				} else {
					$email = __( 'No Email Found!', 'wholesalex-pro' );
				}
				$column_data['email'] = $email;
			}

			$column_data['type']              = get_post_meta( $conversation_id, '__conversation_type', true );
			$column_data['status']            = get_post_meta( $conversation_id, '__conversation_status', true );
			$column_data['edit_conversation'] = get_edit_post_link( $conversation_id, 'edit' );
			$column_data['trash_link']        = get_delete_post_link( $conversation_id );
			$column_value[]                   = $column_data;

		}

		return array(
			'conversations' => $column_value,
			'total'         => $total_conversations,
		);

	}

	public static function get_conversation_types() {
		$types = array(
			'inquiry' => 'Inquiry',
			'message' => 'Message',
		);

		$types = apply_filters( 'wholesalex_conversation_types', $types );

		return $types;
	}

	public static function get_conversation_status() {
		$statuses = array(
			'active'  => 'Active',
			'open'    => 'Open',
			'resolve' => 'Resolved',
		);

		$statuses = apply_filters( 'wholesalex_conversation_statuses', $statuses );

		return $statuses;
	}

	public static function get_post_statuses() {
		$statuses = array(
			'publish'    => __('Publish','wholesalex-pro'),
			'pending'    => __('Pending','wholesalex-pro'),
			'draft'      => __('Draft','wholesalex-pro'),
			'auto-draft' => __('Auto Draft','wholesalex-pro'),
			'future'     => __('Future','wholesalex-pro'),
			'private'    => __('Private','wholesalex-pro'),
			'inherit'    => __('Inherit','wholesalex-pro'),
			'trash'      => __('Trash','wholesalex-pro'),
		);
		return $statuses;

	}
	public static function get_conversation_bulk_action() {
		$actions = array(
			'move_to_trash' => __('Move To Trash','wholesalex-pro'),
			'resolved'      => __('Set Status to Resolve','wholesalex-pro'),
			'open'          => __('Set Status to Open','wholesalex-pro'),
		);

		$actions = apply_filters( 'wholesalex_conversation_bulk_actions', $actions );

		return $actions;
	}

	public static function get_wholesalex_conversation_columns() {
		$columns = array(
			'title'  => __( 'Conversation', 'wholesalex-pro' ),
			'user'   => __( 'User', 'wholesalex-pro' ),
			'email'  => __( 'Email', 'wholesalex-pro' ),
			'type'   => __( 'Type', 'wholesalex-pro' ),
			'status' => __( 'Status', 'wholesalex-pro' ),
		);

		$columns = apply_filters( 'wholesalex_conversation_columns', $columns );

		$columns = wholesalex()->insert_into_array( $columns, array( 'action' => __( 'Action', 'wholesalex-pro' ) ) );

		return $columns;
	}


	public function conversation_page_content() {
		wp_enqueue_script( 'whx_conversation', WHOLESALEX_PRO_URL . 'assets/js/whx_conversation.js', array( 'react', 'react-dom', 'wp-polyfill', 'wp-api-fetch' ), WHOLESALEX_PRO_VER, true );
		wp_enqueue_script( 'wholesalex_node_vendors' );
		wp_enqueue_script( 'wholesalex_components' );

		$heading_data = array();

		$columns = array(
			'title'  => __( 'Conversation', 'wholesalex-pro' ),
			'user'   => __( 'User', 'wholesalex-pro' ),
			'email'  => __( 'Email', 'wholesalex-pro' ),
			'type'   => __( 'Type', 'wholesalex-pro' ),
			'status' => __( 'Status', 'wholesalex-pro' ),
		);
		// Prepare as heading data
		foreach ( self::get_wholesalex_conversation_columns() as $key => $value ) {
			$data               = array();
			$data['all_select'] = '';
			$data['name']       = $key;
			$data['title']      = $value;
			if ( 'action' == $key ) {
				$data['type'] = '3dot';
			} else {
				$data['type'] = 'text';
			}

			$heading_data[ $key ] = $data;
		}

		$heading_data['title']['status']  = 'yes';
		$heading_data['user']['status']   = 'yes';
		$heading_data['status']['status'] = 'yes';
		$heading_data['type']['status']   = 'yes';
		$heading_data['email']['status']  = 'yes';
		$heading_data['action']['status'] = 'yes';

		wp_localize_script(
			'whx_conversation',
			'whx_conversation',
			array(
				'heading'               => $heading_data,
				'bulk_actions'          => wholesalex()->insert_into_array( self::get_conversation_bulk_action(), array( '' => __( 'Bulk Actions', 'wholesalex-pro' ) ), 0 ),
				'statuses'              => wholesalex()->insert_into_array(
					self::get_conversation_status(),
					array( '' => __( 'Select Status', 'wholesalex-pro' ) ),
					0
				),
				'types'                 => wholesalex()->insert_into_array(
					self::get_conversation_types(),
					array( '' => __( 'Select Type', 'wholesalex-pro' ) ),
					0
				),
				'new_conversation_url'  => admin_url( 'post-new.php?post_type=wsx_conversation' ),
				'post_statuses'         => self::get_post_statuses(),
				'conversation_per_page' => 10,
				'i18n' => array(
					'edit' => __('Edit','wholesalex-pro'),
					'resolved' => __('Resolved','wholesalex-pro'),
					'force_delete' => __('Force Delete','wholesalex-pro'),
					'move_to_trash' => __('Move To Trash','wholesalex-pro'),
					'selected_conversations' => __('Selected Conversations','wholesalex-pro'),
					'add_new_conversation' => __('Add New Conversation','wholesalex-pro'),
					'apply' => __('Apply','wholesalex-pro'),
					'columns' => __('Columns','wholesalex-pro'),
					'no_conversations_found' => __('No Conversations Found!','wholesalex-pro'),
					'showing' => __('Showing','wholesalex-pro'),
					'pages' => __('Pages','wholesalex-pro'),
				)
			)
		);

		?>
			<div id="wholesalex_conversation_root"> </div>
		<?php
	}

	/**
	 * Conversation Replay Metabox Callback
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function reply_metabox_callback() {
		add_meta_box(
			'wholesalex_metabox_id',
			__( 'Replies', 'wholesalex-pro' ),
			array( $this, 'metabox_html' ),
			'wsx_conversation'
		);

		add_meta_box(
			'wholesalex_data_id',
			__( 'Data', 'wholesalex-pro' ),
			array( $this, 'metabox_side_html' ),
			'wsx_conversation',
			'side'
		);
	}

	/**
	 * Conversation Side Metabox Content
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function metabox_side_html() {
		wp_nonce_field( 'conversation_metabox_nonce', 'meta_box_nonce' );
		$post_id            = get_the_ID();
		$type               = get_post_meta( $post_id, '__conversation_type', true );
		$status             = get_post_meta( $post_id, '__conversation_status', true );
		$conversation_types = apply_filters(
			'wholesalex_addon_conversation_types',
			array(
				'inquiry' => wholesalex()->get_language_n_text( '_language_conversations_list_type_inquiry', __( 'Inquiry', 'wholesalex-pro' ) ),
				'message' => wholesalex()->get_language_n_text( '_language_conversations_list_type_message', __( 'Message', 'wholesalex-pro' ) ),
			)
		);
		?>

		<div>
			<strong><?php echo esc_html_e( 'Type', 'wholesalex-pro' ); ?></strong>
			<select name="conversation_type">
				<option value=""><?php echo esc_html_e( '- Select -', 'wholesalex-pro' ); ?></option>
				<?php
				foreach ( $conversation_types as $key => $value ) {
					?>
						<option value="<?php echo esc_attr( $key ); ?>" <?php selected( $type, $key ); ?>><?php echo esc_html( $value ); ?></option>
					<?php
				}
				?>
			</select>
			<hr>
			<strong><?php echo esc_html_e( 'Status', 'wholesalex-pro' ); ?></strong>
			<select name="conversation_status">
				<option value=""><?php echo esc_html_e( '- Select -', 'wholesalex-pro' ); ?></option>
				<option value="new" <?php selected( $status, 'new' ); ?>><?php echo esc_html_e( 'New', 'wholesalex-pro' ); ?></option>
				<option value="open" <?php selected( $status, 'open' ); ?>><?php echo esc_html_e( 'Open', 'wholesalex-pro' ); ?></option>
				<option value="resolved" <?php selected( $status, 'resolved' ); ?>><?php echo esc_html_e( 'Resolved', 'wholesalex-pro' ); ?></option>
			</select>
		</div>
			<?php
	}

	/**
	 * Conversation Reply Metabox Save
	 *
	 * @param mixed $post_id Current Conversation ID.
	 * @return void
	 * @since 1.0.0
	 * @since 1.0.2 Preserve New Line In Conversation Content.
	 */
	public function reply_metabox_save_callback( $post_id ) {
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( ! isset( $_POST['meta_box_nonce'] ) || ! wp_verify_nonce( sanitize_key( wp_unslash( $_POST['meta_box_nonce'] ) ), 'conversation_metabox_nonce' ) ) {
			return;
		}
		
		if ( ! current_user_can( 'edit_post', $post_id ) ) {
			return;
		}
		
		if ( isset( $_POST['post_type'] ) && 'wsx_conversation' === $_POST['post_type'] ) {
			
			if ( current_user_can( 'edit_page', $post_id ) ) {
				$user_id     = get_current_user_id();
				$content     = get_the_content( $post_id );
				$new_content = isset( $_POST['wsx-editor'] ) ? sanitize_textarea_field( wp_unslash( $_POST['wsx-editor'] ) ) : '';
				$edit_id     = isset( $_POST['wsxid'] ) ? sanitize_text_field( wp_unslash( $_POST['wsxid'] ) ) : '';
				$author_id   = isset( $_POST['started_by'] ) ? sanitize_text_field( wp_unslash( $_POST['started_by'] ) ) : '';

				do_action( 'wholesalex_before_process_conversation', $_POST );

				if ( ! wp_is_post_revision( $post_id ) ) {
					if ( $edit_id ) {
						if ( $new_content ) {
							if ( has_blocks( $content ) ) {
								$reply_lists = parse_blocks( $content );
								$temp        = array();
								foreach ( $reply_lists as $val ) {
									if ( $val['attrs']['id'] == $edit_id ) {
										$val['innerHTML']    = $new_content;
										$val['innerContent'] = array( $new_content );
										$temp[]              = $val;
									} else {
										$temp[] = $val;
									}
								}
								$data = array(
									'ID'           => $post_id,
									'post_author'  => $author_id,
									'post_content' => serialize_blocks( $temp ),
								);
								$this->save_data( $data );
							}
						} elseif ( $author_id ) {
							$data = array(
								'ID'          => $post_id,
								'post_author' => $author_id,
							);
							$this->save_data( $data );
						}
					} else {
						if ( $new_content ) {
							$reply_id = (int) get_post_meta($post_id,'wholesalex_conversation_reply_id',true);
							$reply_id = $reply_id+1;
							$data = array(
								'ID'           => $post_id,
								'post_content' => $content . '<!-- wp:wsx_reply {"id":"' . $reply_id . '","author":"' . $user_id . '", "time":"' . gmdate( 'Y-m-d h:i:s A' ) . '"} -->' . nl2br( $new_content ) . '<!-- /wp:wsx_reply -->',
							);
							if ( $author_id ) {
								$data['post_author'] = $author_id;
							}
							$this->save_data( $data );
							update_post_meta($post_id,'wholesalex_conversation_reply_id',$reply_id);
							if ( $author_id ) {
								$user_data             = get_userdata( $author_id );
								$user_email            = $user_data->user_email;
								do_action( 'wholesalex_conversation_create_back_end', $post_id , $author_id, $user_email);
							}else{
								$post_author_id = get_post_field('post_author',  get_the_ID() );
								$user_data             = get_userdata( $post_author_id );
								$user_email            = $user_data->user_email;
								do_action( 'wholesalex_conversation_reply_back_end', $post_id , $post_author_id, $user_email);
							}
						}
					}

					if ( isset( $_POST['conversation_type'] ) && isset( $_POST['conversation_status'] ) ) {
						$type   = sanitize_text_field( wp_unslash( $_POST['conversation_type'] ) );
						$status = sanitize_text_field( wp_unslash( $_POST['conversation_status'] ) );
						update_post_meta( $post_id, '__conversation_type', $type );
						update_post_meta( $post_id, '__conversation_status', $status );
					}
				}
			}
		}
	}

	/**
	 * Save Conversation
	 *
	 * @param array $data Conversation Data.
	 * @return void
	 * @since 1.0.0
	 */
	public function save_data( $data ) {
		remove_action( 'save_post', array( $this, 'reply_metabox_save_callback' ) ); // For Unlimited Loop.
		wp_update_post( $data );
		add_action( 'save_post', array( $this, 'reply_metabox_save_callback' ) ); // Re hook Save Function.
	}

	/**
	 * Conversation Metabox Content
	 *
	 * @return void
	 * @since 1.0.0
	 * @since 1.0.2 Preserve New Line In Conversation Content.
	 */
	public function metabox_html() {
		wp_nonce_field( 'conversation_metabox_nonce', 'meta_box_nonce' );

		do_action( 'wholesalex_conversation_metabox_content' );

		$edit_id     = '';
		$post_id     = get_the_ID();
		$user_id     = get_current_user_id();
		$reply_lists = get_the_content( $post_id );

		// Conversation Created By.
		$started_by = get_post_field( 'post_author', $post_id );

		$status    = get_post_meta( $post_id, '__conversation_type', true );
		$is_quote  = ( 'quote' === $status );
		$username  = '';
		$email     = '';
		$user_info = '';
		if ( $is_quote && ! $started_by ) {
			$quote_data = get_post_meta( $post_id, '__wholesalex_quote_data', true );
			$quote_data = json_decode( $quote_data, true );
			$username   = isset( $quote_data['name'] ) ? $quote_data['name'] . __('(Guest)','wholesalex-pro') : __('Guest','wholesalex-pro');
			$email      = isset( $quote_data['user_email'] ) ? $quote_data['user_email'] : '';
		} else {
			if ( $started_by ) {
				$user_info = get_userdata( $started_by );
				$username  = $user_info ? $user_info->display_name : __( 'Deleted User', 'wholesalex-pro' );
				$email     = $user_info ? $user_info->user_email : __( 'Deleted User', 'wholesalex-pro' );
			}
		}

		if ( ! isset( $_GET['post'] ) ) { //phpcs:ignore
			echo '<input type="hidden" name="started_by" value="' . esc_html( $started_by ) . '">';
			echo '<span>' . esc_html__( 'Conversation Creator:', 'wholesalex-pro' ) . '</span>';
			echo '<div class="wsx-search-label"></div>';
			echo '<input type="text" name="search_user" placeholder="search..." class="wsx-search-box">';
			echo '<ul class="wsx-search-result"></ul>';
		} else {
			echo '<div class="wsx-created-by">';
			echo '<div class="wsx-created-author">' . get_avatar( $user_info ? $user_info->ID : '', 44 ) . '</div>';
			echo '<div class="wsx-created-content">';
			echo '<div>' . esc_html__( 'Name', 'wholesalex-pro' ) . ': ' . esc_html( $username ) . '</div>';
			echo '<div>' . esc_html__( 'Email', 'wholesalex-pro' ) . ': ' . esc_html( $email ) . '</div>';
			echo '</div>';
			echo '</div>';
		}

		if ( has_blocks( $reply_lists ) ) {
			$reply_lists = parse_blocks( $reply_lists );

			// Delete Action.
			if ( isset( $_GET['wsxdel'] ) ) { //phpcs:ignore
				$action = sanitize_key( $_GET['wsxdel'] ); //phpcs:ignore
				if ( current_user_can( 'edit_page', $post_id ) && $action ) {
					$temp = array();
					foreach ( $reply_lists as $val ) {
						if ( $val['attrs']['id'] != $action ) {
							$temp[] = $val;
						}
					}
					$data = array(
						'ID'           => $post_id,
						'post_content' => serialize_blocks( $temp ),
					);
					$this->save_data( $data );
					$reply_lists = $temp;
				}
			}

			// Edit Action.
			if ( isset( $_GET['wsxid'] ) ) {
				$action = sanitize_key( $_GET['wsxid'] );
				$temp   = array();
				foreach ( $reply_lists as $val ) {
					if ( $val['attrs']['id'] == $action ) {
						$edit_id = $val['attrs']['id'];
					}
				}
			}

			foreach ( $reply_lists as $reply ) {
				$user       = get_userdata( $reply['attrs']['author'] );
				$reply_user = $user ? $user->display_name : __( 'Deleted User', 'wholesalex-pro' );

				if ( $is_quote && ! $started_by ) {
					$reply_user = $username;
				}
				$reply_user = apply_filters( 'wholesalex_addon_conversation_reply_display_name', $reply_user, $user ? $user->ID : 0, $started_by );
				echo '<div class="wsx-reply-lists ' . ( $reply['attrs']['author'] == $user_id ? 'wsx-reply-right' : 'wsx-reply-left' ) . '">';
				echo '<div class="wsx-reply-meta">';
				echo '<span class="wsx-name">' . wp_kses_post($reply_user) . '</span> | ';
				echo '<span class="wsx-time">' . wp_kses_post( human_time_diff( strtotime( $reply['attrs']['time'] ), time() ) ) . ' ago</span> | ';
				echo '<span class="wsx-date">' . wp_kses_post( get_date_from_gmt( $reply['attrs']['time'], 'Y-m-d h:i:s A' ) ) . '</span>';
				echo '</div>';

				echo '<div class="wsx-content">';
				/**
				 * Preserve Conversation Content New Line
				 *
				 * @since 1.0.2
				 */
				do_action('wholesalex_addon_conversation_before_message_content',$post_id,$reply);
				
				$output = $reply['innerHTML'];
				echo wp_kses(
					$output,
					array(
						'br'     => array(),
						'a'      => array( 'href' => array() ),
						'div'    => array(
							'class' => array(),
							'id'    => array(),
						),
						'span'   => array(
							'class' => array(),
							'id'    => array(),
						),
						'button' => array(
							'class' => array(),
							'id'    => array(),
						),
						'table'  => array(
							'class' => array(),
							'id'    => array(),
						),
						'tbody'  => array(),
						'thead'  => array(),
						'tr'     => array(),
						'td'     => array(),
						'th'     => array(),
					)
				);

				if ( is_object($user) && $user->ID == get_current_user_id() ) {

					echo '<div class="wsx-content-btn">';
					echo '<a class="wholesalex-btn-extra-sm wholesalex-btn-danger" href="' . esc_url( add_query_arg( 'wsxdel', $reply['attrs']['id'] ) ) . '">' . esc_html__( 'Delete', 'wholesalex-pro' ) . '</a>';
					// echo '<a class="wholesalex-btn-extra-sm wholesalex-btn-black" href="' . esc_url( add_query_arg( 'wsxid', $reply['attrs']['id'] ) ) . '">' . esc_html__( 'Edit', 'wholesalex-pro' ) . '</a>';
					echo '</div>';
				}
				echo '</div>';
				echo '</div>';
			}
		}

		?>
		<div class="wsx-conversation-wrap">
			<?php
			if ( $edit_id ) {
				echo '<input type="hidden" name="wsxid" value="' . esc_attr( $edit_id ) . '">';
			}
			if ( $started_by ) {
				echo '<textarea name="wsx-editor"></textarea>';
				echo '<input type="hidden" id="token" name="token" value=""/>';
			} else {
				$message = apply_filters( 'wholesalex_addon_conversation_delete_user_notice', __( 'The user has been deleted.You cannot reply this conversation.', 'wholesalex-pro' ), $status );
				?>
					<div class="wholesalex_conversation_notice">
						<?php echo esc_html( $message ); ?>
					</div>
				<?php
			}
			if ( $started_by ) {
				?>
				<div class="wholesalex_conversation_buttons">
					<?php do_action( 'wholesalex_edit_conversation_before_send_message_button', $post_id ); ?>
					<button id="wholesalex_conversation_submit" class="wsx-reply wholesalex-btn wholesalex-btn-primary"><?php echo esc_html__( 'Send Message', 'wholesalex-pro' ); ?></button>
					<?php do_action( 'wholesalex_edit_conversation_after_send_message_button', $post_id ); ?>
				</div>
				<?php
			} else {
				?>
					<div class="wholesalex_conversation_buttons">
						<?php do_action( 'wholesalex_edit_conversation_before_send_message_button', $post_id ); ?>
					</div>
				<?php
			}

			?>
		</div>
			<?php
			if ( 'yes' === wholesalex()->get_setting( 'wsx_addon_recaptcha' ) ) {
					$__site_key = wholesalex()->get_setting( '_settings_google_recaptcha_v3_site_key' );
				?>
			<script type="text/javascript">
				jQuery("#wholesalex_conversation_submit").click(e => {
					e.preventDefault();
					grecaptcha.ready(function() {
						grecaptcha.execute('<?php echo esc_html( $__site_key ); ?>', {
							action: 'submit'
						}).then(function(token) {
							document.getElementById("token").value = token;
							jQuery("#wholesalex_conversation_submit").unbind('click').submit();
							jQuery("#wholesalex_conversation_submit").click();

						});
					});
				})
			</script>
				<?php
			}
	}

	/**
	 * Conversation Settings Fields
	 *
	 * @param array $sections Setting Sections.
	 */
	public function conversation_settings_fields( $sections ) {
		$admin_email = get_option('admin_email');
		if ( function_exists( 'wholesalex' ) ) {
			$__conversation_section = array(
				'conversation' => array(
					'label' => __( 'Conversations', 'wholesalex-pro' ),
					'attr'  => array(
						'_settings_show_conversation_my_account_page' => array(
							'type'    => 'switch',
							'label'   => __( 'Show Conversation', 'wholesalex-pro' ),
							'desc'    => __( 'Show conversation option to my account page.', 'wholesalex-pro' ),
							'help'    => '',
							'default' => 'yes',
						),
						'_conversation_my_account_endpoint' => array(
							'type'        => 'text',
							'label'       => __( 'Conversation My Account Page Endpoint', 'wholesalex-pro' ),
							'placeholder' => '',
							'help'        => '',
							'default'     => 'wsx-conversation',
						),
						'_conversation_my_account_endpoint_title'     => array(
							'type'        => 'text',
							'label'       => __( 'Conversation My Account Endpoint Title', 'wholesalex-pro' ),
							'placeholder' => '',
							'help'        => '',
							'default'     => __( 'Conversation', 'wholesalex-pro' ),
						),
						'_conversation_admin_emails'     => array(
							'type'        => 'text',
							'label'       => __( 'Admin Email List', 'wholesalex-pro' ),
							'placeholder' => '',
							'help'        => 'Multiple admin email addresses can be set (separated by comma) for receiving notifications',
							'default'     => $admin_email,
						),
					),
				),
			);

			/**
			 * Conversations Language and Text Settings Field
			 *
			 * @since 1.0.2
			 */
			if ( isset( $sections['language_n_text']['attr'] ) && is_array( $sections['language_n_text']['attr'] ) ) {
				$__fields                            = array(
					'_language_conversation_go_back'     => array(
						'type'        => 'text',
						'label'       => __( 'Go Back Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Go back', 'wholesalex-pro' ),
					),
					'_language_conversation_x_time_ago'  => array(
						'type'        => 'text',
						'label'       => __( 'Conversation X Time Ago Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'ago', 'wholesalex-pro' ),
					),
					'_language_conversation_reply'       => array(
						'type'        => 'text',
						'label'       => __( 'Conversation Reply Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Reply', 'wholesalex-pro' ),
					),
					'_language_conversation_error'       => array(
						'type'        => 'text',
						'label'       => __( 'Conversation Error Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Something Went Wrong!', 'wholesalex-pro' ),
					),
					'_language_conversation_heading'     => array(
						'type'        => 'text',
						'label'       => __( 'Conversation Heading Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Conversations', 'wholesalex-pro' ),
					),
					'_language_new_conversation'         => array(
						'type'        => 'text',
						'label'       => __( 'New Conversation Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'New Conversation', 'wholesalex-pro' ),
					),
					'_language_conversations_list_title' => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Title Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Title', 'wholesalex-pro' ),
					),
					'_language_conversations_list_type'  => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Type Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Type', 'wholesalex-pro' ),
					),
					'_language_conversations_list_type_inquiry' => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Type Inquiry Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Inquiry', 'wholesalex-pro' ),
					),
					'_language_conversations_list_type_message' => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Type Message Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Message', 'wholesalex-pro' ),
					),
					'_language_conversations_list_type_quote' => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Type Quote Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Quote', 'wholesalex-pro' ),
					),
					'_language_conversations_last_reply_time' => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Last Reply Time Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Last Reply Time', 'wholesalex-pro' ),
					),
					'_language_conversations_action'     => array(
						'type'        => 'text',
						'label'       => __( 'Conversations Action Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Action', 'wholesalex-pro' ),
					),
					'_language_start_conversation'       => array(
						'type'        => 'text',
						'label'       => __( 'Start Conversation Text', 'wholesalex-pro' ),
						'placeholder' => '',
						'help'        => '',
						'default'     => __( 'Start Conversation', 'wholesalex-pro' ),
					),
				);
				$sections['language_n_text']['attr'] = array_merge( $sections['language_n_text']['attr'], $__fields );
			}

			$sections = wholesalex()->insert_into_array( $sections, $__conversation_section, 3 );
		}

		return $sections;
	}

	/**
	 * Conversations Default Sort by Last Modified/Reply Time.
	 *
	 * @param WP_QUERY $wp_query wp query.
	 * @return void
	 * @since 1.0.2
	 */
	public function sort_by_last_reply_time( $wp_query ) {
		global $pagenow;
		if ( current_user_can( 'edit_posts' ) && is_admin() && 'edit.php' === $pagenow ) {
			$post_type = $wp_query->query['post_type'];
			if ( 'wsx_conversation' === $post_type ) {
				$wp_query->set( 'orderby', 'modified' );
				$wp_query->set( 'order', 'DESC' );
			}
		}
	}


	/**
	 * Get Open Conversation Count.
	 *
	 * @return int
	 * @since 1.0.5
	 */
	public function get_open_conversation_count() {
		global $wpdb;
		$count = $wpdb->get_var( "SELECT count(*) from $wpdb->postmeta where meta_key='__conversation_status' and meta_value='open';" );
		return (int) $count;
	}
}
