<?php
/**
 * Class wholesalex new user Conversation reply notification file.
 *
 * @package WHOLESALEX
 * @since 1.4.4
 */

namespace WHOLESALEX_PRO;

use WHOLESALEX\WHOLESALEX_Email_Manager;

use WC_Email;
use WP_User;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}


/**
 * Customer New Account.
 *
 * An email sent to the Admin when they create an New Conversation.
 *
 * @extends     WC_Email
 */
class WholesaleX_New_Conversation_Reply_Email extends WC_Email {

     /**
     * User Conversation Id.
     *
     * @var string
     */
    public $conversation_id;

    /**
     * User login name.
     *
     * @var string
     */
    public $user_login;

    /**
     * User email.
     *
     * @var string
     */
    public $user_email;

    /**
     * User password.
     *
     * @var string
     */
    public $user_pass;

    /**
     * User parent.
     *
     * @var string
     */
    public $parent_user_id;

    /**
     * Is the password generated?
     *
     * @var bool
     */
    public $password_generated;

    /**
     * Magic link to verify email.
     *
     * @var string
     */
    public $confirmation_url;

    /**
     * Constructor.
     */
    public function __construct() {
        $this->id             = 'wholesalex_conversation_reply_email';
        $this->title          = __( 'WholesaleX: Reply to Conversation Alert Email ', 'wholesalex-pro' );
        $this->description    = __( 'WholesaleX: When users or Admin reply any message to the admin or user then admin or user get email notifications', 'wholesalex-pro' );
        $this->template_base  = WHOLESALEX_PRO_PATH.'addons/conversation/emails/templates/';
        $this->template_html  = 'wholesalex-conversation-reply-email.php';
        $this->placeholders = apply_filters('wholesalex_conversation_admin_smart_tags',array(
            	'{date}'                => gmdate( 'd-m-Y' ),
            	'{admin_name}'          => WHOLESALEX_Email_Manager::get_admin_display(),
            	'{site_name}'           => get_option( 'blogname' ),
            	'{conversation_name}'     => '',
            	'{conversation_email}'     => '',
        ));

        // Call parent constructor.

        parent::__construct();
    }

    /**
     * Get email subject.
     *
     * @since  1.4.4
     * @return string
     */
    public function get_default_subject() {
        return __( 'You’ve received a reply', 'wholesalex-pro' );
    }

    /**
     * Get email heading.
     *
     * @since  1.4.4
     * @return string
     */
    public function get_default_heading() {
        return __( 'You have a New Reply Message', 'wholesalex-pro' );
    }

    /**
     * Trigger.
     *
     * @param int    $user_id User ID.
     * @since  1.4.4
     */
    public function trigger( $post_id, $user_id, $user_emails ) {
       
        $this->setup_locale();
        if ( $user_id ) {
            $this->conversation_id = $post_id;
            $this->recipient = $user_emails;
            $this->object = new WP_User( $user_id );
            $this->user_login         = stripslashes( $this->object->user_login );
            $this->user_email         = stripslashes( $this->object->user_email );
            $this->placeholders['{admin_name}'] = WHOLESALEX_Email_Manager::get_admin_display();             
            $this->placeholders['{date}'] = gmdate( 'd-m-Y' );             
            $this->placeholders['{site_name}'] = get_option( 'blogname' );             
            $this->placeholders['{conversation_name}'] = $this->user_login;             
            $this->placeholders['{conversation_email}'] = $this->user_email;             
        }
        if ( $this->is_enabled() && $this->get_recipient() ) {
            $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
        }
        $this->restore_locale();
    }

    /**
     * Get content html.
     *
     * @return string
     */
    public function get_content_html() {
        return wc_get_template_html(
            $this->template_html,
            array(
                'email_heading'      => $this->get_heading(),
                'additional_content' => $this->get_additional_content(),
                'user_login'         => $this->user_login,
                'user_pass'          => $this->user_pass,
                'parent_id'          => $this->parent_user_id,
                'conversation_id'    => $this->conversation_id,
                'blogname'           => $this->get_blogname(),
                'sent_to_admin'      => false,
                'plain_text'         => false,
                'email'              => $this,
            ),
            get_stylesheet_directory().'/wholesalex/', $this->template_base
        );
    }

    /**
     * Default content to show below main email content.
     *
     * @since  1.4.4
     * @return string
     */
    public function get_default_additional_content() {
        return __( '', 'wholesalex-pro' );
    }

    /**
 * Initialise Settings Form Fields - these are generic email options most will use.
 */
public function init_form_fields() {
    /* translators: %s: list of placeholders */
    $placeholder_text  = sprintf( __( 'Available placeholders: %s', 'wholesalex-pro' ), '<code>' . esc_html( implode( '</code>, <code>', array_keys( $this->placeholders ) ) ) . '</code>' );
    $this->form_fields = array(
        'enabled'            => array(
            'title'   => __( 'Enable/Disable', 'wholesalex-pro' ),
            'type'    => 'checkbox',
            'label'   => __( 'Enable this email notification', 'wholesalex-pro' ),
            'default' => 'no',
        ),
        'subject'            => array(
            'title'       => __( 'Subject', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email subject. To change default email subject, makes changes here.',
            'placeholder' => $this->get_default_subject(),
            'default'     => '',
        ),
        'heading'            => array(
            'title'       => __( 'Email heading', 'wholesalex-pro' ),
            'type'        => 'text',
            'desc_tip'    => true,
            'description' => 'This control the email heading. To change default email heading, makes changes here.',
            'placeholder' => $this->get_default_heading(),
            'default'     => '',
        ),
        'additional_content' => array(
            'title'       => __( 'Additional content', 'wholesalex-pro' ),
            'description' => __( 'Text to appear below the main email content.', 'wholesalex-pro' ) . ' ' . $placeholder_text,
            'css'         => 'width:400px; height: 75px;',
            'placeholder' => __( 'N/A', 'wholesalex-pro' ),
            'type'        => 'textarea',
            'default'     => $this->get_default_additional_content(),
            'desc_tip'    => true,
        ),
        'email_type'         => array(
            'title'       => __( 'Email type', 'wholesalex-pro' ),
            'type'        => 'select',
            'description' => __( 'Choose which format of email to send.', 'wholesalex-pro' ),
            'default'     => 'html',
            'class'       => 'email_type wc-enhanced-select',
            'options'     => array(
                'html'      => __( 'HTML', 'woocommerce' ),
                'multipart' => __( 'Multipart', 'woocommerce' )
            ),
            'desc_tip'    => true,
        ),
    );
}
}



return new WholesaleX_New_Conversation_Reply_Email();
