<?php
/**
 * WholesaleX Payment Gateway
 *
 * @package WHOLESALEX
 * @since 1.0.0
 */

namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
use WC_Payment_Gateway;
use Exception;
if ( class_exists( 'WC_Payment_Gateway' ) ) {
	/**
	 * WholesaleX Payment Gateway
	 */
	class WHOLESALEX_Payment_Gateway extends WC_Payment_Gateway {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->initial_setup();

			// Load the settings.
			$this->init_form_fields();
			$this->init_settings();
			// Get settings.
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->instructions = $this->get_option( 'instructions' );

			$this->supports = array(
				'products',
				'refunds',
			);

			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_pre_payment_complete', array( $this, 'woocommerce_pre_payment_complete' ) );
		}

		/**
		 * WholesaleX Payment Gateway Initial Properties
		 *
		 * @since 1.0.0
		 */
		private function initial_setup() {

			$this->id                 = 'wholesalex';
			$this->method_title       = apply_filters('wholesalex_wallet_addon_payment_gateway_title',__( 'WholesaleX Wallet', 'wholesalex-pro' ));
			$this->method_description = apply_filters( 'wholesalex_wallet_addon_payment_gateway_description', __( 'Enable the WholesaleX Wallet to let the customers add funds via the wallet and use it for future payments.', 'wholesalex-pro' ));
			$this->has_fields         = false;
		}

		/**
		 * WholesaleX Payment Gateway Settings Form Fields Initialize
		 *
		 * @since 1.0.0
		 */
		public function init_form_fields() {
			$this->form_fields = array(
				'enabled'      => array(
					'title'       => __( 'Enable/Disable', 'wholesalex-pro' ),
					'label'       => __( 'Enable Wallet Balance Payment Gateway', 'wholesalex-pro' ), 
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no',
				),
				'title'        => array(
					'title'       => __( 'Title', 'wholesalex-pro' ),
					'type'        => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'wholesalex-pro' ),
					'default'     => __( 'WholesaleX Wallet Payment', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
				'description'  => array(
					'title'       => __( 'Description', 'wholesalex-pro' ),
					'type'        => 'textarea',
					'description' => __( 'Payment method description that the customer will see on your checkout.', 'wholesalex-pro' ),
					'default'     => __( 'Pay with WholesaleX Credit Balance', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
				'instructions' => array(
					'title'       => __( 'Instructions', 'wholesalex-pro' ),
					'type'        => 'textarea',
					'description' => __( 'Instructions that will be added to the thank you page.', 'wholesalex-pro' ),
					'default'     => __( 'Pay with WholesaleX Credit Balance', 'wholesalex-pro' ),
					'desc_tip'    => true,
				),
			);
		}

		/**
		 * Is wholesalex wallet payment gateway available
		 *
		 * @return boolean
		 * @since 1.1.9
		 */
		public function is_available() {
			if ( is_checkout() ) {
				return apply_filters( 'wholesalex_wallet_payment_gateway_available', ( parent::is_available() && is_user_logged_in() ) );
			}
			return parent::is_available();
		}

		/**
		 * Set Current WholesaleX Balance as Payment Gateways Icon
		 *
		 * @since 1.0.0
		 * @since 1.0.9 Add wholesalex_wallet_user_id filter
		 * @since 1.0.10 Add _language_wholesalex_payment_gateway_icon_text Settings. User can change wholesalex payment gateway icon.
		 */
		public function get_icon() {
			$user_id         = apply_filters( 'wholesalex_wallet_user_id', get_current_user_id() );
			$current_balance = wholesalex_wallet()->get_wholesalex_balance( $user_id );
			/* translators: %s: Current WholesaleX Balance */
			$icon_text = apply_filters( 'wholesalex_wallet_gateway_icon', sprintf( __( 'Remaining WholesaleX Balance: <strong>%s</strong>', 'wholesalex-pro' ), $current_balance ), $this->id );
			if ( wholesalex()->get_setting( '_language_wholesalex_payment_gateway_icon_text' ) ) {
				$icon_text = wholesalex()->get_setting( '_language_wholesalex_payment_gateway_icon_text' );
				$icon_text = str_replace( '{remaining_balance}', sprintf( '<strong>%s</strong>', $current_balance ), $icon_text );
			}
			return $icon_text;
		}

		/**
		 * Process WholesaleX Balance Payment
		 *
		 * @param int $order_id Order Id.
		 * @return array process payment results.
		 * @since 1.0.0
		 * @since 1.0.9 Add wholesalex_wallet_user_id filter
		 */
		public function process_payment( $order_id ) {
			$order          = wc_get_order( $order_id );
			$order_subtotal = $order->get_total( 'edit' );
			$user_id        = apply_filters( 'wholesalex_wallet_user_id', get_current_user_id() );

			if ( $order_subtotal > wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' ) ) {
				wc_add_notice( __( 'Insufficient Balance. Please TopUP balance before user Balance Gateway', 'wholesalex-pro' ), 'error' );
				return;
			}

			// Reduce stock levels.
			wc_reduce_stock_levels( $order_id );

			// Remove cart.
			WC()->cart->empty_cart();

			// Complete order payment.
			$order->payment_complete();

			return array(
				'result'   => 'success',
				'redirect' => $this->get_return_url( $order ),
			);
		}

		/**
		 * Debit user wallet on WooCommerce payment complete.
		 *
		 * @param WC_Order $order_id Order ID.
		 * @throws Exception Exception.
		 * @since 1.0.0
		 * @since 1.0.9 add wholesalex_wallet_user_id filter.
		 */
		public function woocommerce_pre_payment_complete( $order_id ) {
			$order = wc_get_order( $order_id );
			if ( 'wholesalex' === $order->get_payment_method( 'edit' ) && ! $order->get_transaction_id( 'edit' ) && $order->has_status( apply_filters( 'woocommerce_valid_order_statuses_for_payment_complete', array( 'on-hold', 'pending', 'failed', 'cancelled' ), $order ) ) ) {
				$user_id = apply_filters( 'wholesalex_wallet_user_id', $order->get_customer_id( 'edit' ) );
				if ( wholesalex_wallet()->get_wholesalex_balance( $user_id, 'edit' ) >= $order->get_total( 'edit' ) ) {
					$wallet_response = wholesalex_wallet()->initiate_transaction( $user_id, $order->get_total( 'edit' ), 'debit', apply_filters( 'wholesalex_wallet_order_payment_description', __( 'For order payment #', 'wholesalex-pro' ) . $order->get_order_number(), $order ) );
					if ( $wallet_response ) {
						$order->set_transaction_id( $wallet_response );
						do_action( 'wholesalex_payment_processed', $order_id, $wallet_response );
						$order->save();
					} else {
						throw new Exception( __( 'Something went wrong with processing payment please try again.', 'wholesalex-pro' ) ); //phpcs:ignore
					}
				} else {
					throw new Exception( __( 'Insufficient balance', 'wholesalex-pro' ) ); //phpcs:ignore
				}
			}
		}

		/**
		 * Process Refund
		 *
		 * @param  int    $order_id Order ID.
		 * @param  float  $amount Refund amount.
		 * @param  string $reason Refund reason.
		 * @throws Exception Exception.
		 * @return bool|WP_Error
		 * @since 1.0.0
		 * @since 1.0.9 add wholesalex_wallet_user_id filter.
		 */
		public function process_refund( $order_id, $amount = null, $reason = '' ) {
			$order          = wc_get_order( $order_id );
			$user_id        = apply_filters( 'wholesalex_wallet_user_id', $order->get_customer_id() );
			$refund_reason  = $reason ? $reason : apply_filters('wholesalex_wallet_addon_refund_reason_text', __( 'Wallet refund #', 'wholesalex-pro' )) . $order->get_order_number(); //phpcs:ignore
			$transaction_id = wholesalex_wallet()->initiate_transaction( $user_id, $amount, 'credit', $refund_reason );
			if ( ! $transaction_id ) {
				throw new Exception( __( 'Refund not credited to customer', 'wholesalex-pro' ) ); //phpcs:ignore
			}
			do_action( 'wholesalex_wallet_balance_refunded', $order, $amount, $transaction_id );
			return true;
		}
	}
}
