<?php
/**
 * WholesaleX Wallet
 *
 * @package WHOLESALEX
 * @since 1.0.0
 */
namespace WHOLESALEX_PRO;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
/**
 * WholesaleX Wallet Class
 */
final class WholesaleX_Wallet {

	/**
	 * The single instance of the class.
	 *
	 * @var WholesaleX_Wallet
	 * @since 1.0.0
	 */
	protected static $_instance = null;
	/**
	 * Holds Current Balance
	 *
	 * @var WholesaleX_Current_Balance
	 * @since 1.0.0
	 */
	public $wholesalex_balance = 0;

	/**
	 * User ID
	 *
	 * @var int
	 */
	public $user_id = 0;

	/**
	 * Main instance
	 *
	 * @return class object
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Constructor
	 *
	 * @since 1.0.9 add wholesalex_addon_wallet_user_id filter.
	 */
	public function __construct() {
		$this->user_id = apply_filters( 'wholesalex_addon_wallet_user_id', get_current_user_id() );
	}

	/**
	 * Set User ID
	 *
	 * @param int $user_id User ID.
	 */
	private function set_user_id( $user_id = '' ) {
		$this->user_id = $user_id ? $user_id : $this->user_id;
	}

	/**
	 * Get Total Debit Amount
	 *
	 * @param int $user_id User ID.
	 * @return mixed
	 */
	public function get_total_debit( $user_id = '' ) {
		$_total_debit = 0;
		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}

		$transactions = get_user_meta( $user_id, '__wholesalex_transactions_details', true );
		if ( empty( $transactions ) ) {
			return $_total_debit;
		}

		foreach ( $transactions as $transaction ) {
			if(is_numeric($transaction['amount'])) {
				$_total_debit += 'debit' === $transaction['type'] ?  $transaction['amount'] : 0;
			}
		}
		return $_total_debit;
	}

	/**
	 * Get Total Credit Amount
	 *
	 * @param int $user_id User ID.
	 * @return mixed
	 */
	public function get_total_credit( $user_id = '' ) {
		$_total_credit = 0;
		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}

		$transactions = get_user_meta( $user_id, '__wholesalex_transactions_details', true );
		if ( empty( $transactions ) ) {
			return $_total_credit;
		}

		foreach ( $transactions as $transaction ) {
			if(is_numeric( $transaction['amount'])) {
				$_total_credit += 'credit' === $transaction['type'] ? $transaction['amount'] : 0;
			}
		}
		return $_total_credit;
	}


	/**
	 * Get WholesaleX Remaining Credit Limit/Balance
	 *
	 * @param int    $user_id User ID.
	 * @param string $context Contex. view|edit.
	 * @return mixed
	 */
	public function get_wholesalex_balance( $user_id = '', $context = 'view' ) {
		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}
		$this->set_user_id( $user_id );
		$this->wholesalex_balance = 0;
		if ( $this->user_id ) {
			$balance                  = $this->get_total_credit( $this->user_id ) - $this->get_total_debit( $this->user_id );
			$this->wholesalex_balance = apply_filters( 'wholesalex_current_balance', $balance, $this->user_id );
		}
		return 'view' === $context ? wc_price( $this->wholesalex_balance ) : number_format( $this->wholesalex_balance, wc_get_price_decimals(), '.', '' );
	}

	/**
	 * Initiate Transaction
	 *
	 * @param int    $user_id User ID.
	 * @param float  $amount Transaction Amount.
	 * @param string $type Transaction Type . debit|credit.
	 * @param string $reason Purpose of Transaction.
	 * @return int For a successfull Transaction Return Transaction ID, for any failure return 0/false;
	 * @since 1.0.0
	 */
	public function initiate_transaction( $user_id = '', $amount = 0, $type = '', $reason = '' ) {
		if ( ( empty( $user_id ) || ! ( 'credit' === $type || 'debit' === $type ) ) ) {
			return false;
		}

		$this->set_user_id( $user_id );

		return $this->save_transaction( $amount, $type, $reason );
	}


	/**
	 * Save Transaction
	 *
	 * @param int    $amount Transaction Amount.
	 * @param string $type Transaction type.
	 * @param string $reason Transaction Reason.
	 * @return string $transaction_id
	 */
	public function save_transaction( $amount, $type, $reason ) {
		if ( ! $this->user_id ) {
			return false;
		}
		$amount  = max( $amount, 0 );
		$balance = floatval( $this->get_wholesalex_balance( $this->user_id, 'edit' ) );

		$balance = 'credit' === $type ? $balance += floatval( $amount ) : $balance -= floatval( $amount );

		$transactions = get_user_meta( $this->user_id, '__wholesalex_transactions_details', true );
		if ( empty( $transactions ) ) {
			$transactions = array();
		}
		$_id            = uniqid( $this->user_id );
		$transactions[] = array(
			'_id'       => $_id,
			'amount'    => $amount,
			'type'      => $type,
			'reason'    => $reason,
			'timestamp' => time(),
		);
		update_user_meta( $this->user_id, '__wholesalex_transactions_details', $transactions );
		update_user_meta( $this->user_id, '__wholesalex_balance', $balance );
		do_action( 'wholesalex_transaction_complete', $_id, $this->user_id, $type, $amount, $reason );

		return $_id;
	}


	/**
	 * Get WholesaleX Rechargeable Order Ids.
	 *
	 * @return array
	 * @since 1.0.1
	 */
	public function get_rechargeable_order_ids( $args = array() ) {
		$default_args = array(
			'posts_per_page'   => -1,
			'meta_key'         => '_wholesalex_topup_success',
			'meta_value'       => true,
			'post_type'        => 'shop_order',
			'post_status'      => array( 'completed', 'processing', 'on-hold' ),
			'suppress_filters' => true,
		);
		$args         = wp_parse_args( $args, $default_args );
		$orders       = get_posts( $args );
		return wp_list_pluck( $orders, 'ID' );
	}

	/**
	 * Get Wallet Rechargeable Product
	 *
	 * @since 1.0.1
	 */
	public function get_wallet_rechargeable_product() {
		$__id = get_option( '__wholesalex_wallet_topup_product' );
		return wc_get_product( $__id );
	}

	/**
	 * Send Balance Request
	 *
	 * @param string  $from From User ID: The user who send the request.
	 * @param string  $to To User ID: The user who will receive the request.
	 * @param integer $amount Requested Amount.
	 * @param string  $reason Reason For this request
	 * @return void
	 * @since 1.1.4
	 */
	public function send_balance_request( $from = '', $to = '', $amount = 0, $reason = '' ) {
		if ( empty( $from ) || empty( $to ) || empty( $amount ) ) {
			return false;

		}

		// From
		$new_request       = array(
			'id'        => uniqid( 'req' . $from ),
			'from'      => $from,
			'to'        => $to,
			'amount'    => $amount,
			'reason'    => $reason,
			'status'    => 'pending',
			'timestamp' => date( 'U' ),
		);
		$existing_requests = get_user_meta( $from, '__wholesalex_wallet_balance_requests', true );
		if ( ! is_array( $existing_requests ) ) {
			$existing_requests = array();
		}
		$existing_requests[] = $new_request;
		update_user_meta( $from, '__wholesalex_wallet_balance_requests', $existing_requests );

		// To
		$existing_to_requests = get_user_meta( $to, '__wholesalex_wallet_balance_requests', true );
		if ( ! is_array( $existing_to_requests ) ) {
			$existing_to_requests = array();
		}
		$existing_to_requests[] = $new_request;
		update_user_meta( $to, '__wholesalex_wallet_balance_requests', $existing_to_requests );

		return true;

	}

	/**
	 * Approve Balance Reqeust
	 *
	 * @param string $request_id Request ID.
	 * @return void
	 * @since 1.1.4
	 */
	public function approve_balance_request( $request_id = '' ) {
		$balance_requests     = get_user_meta( $this->user_id, '__wholesalex_wallet_balance_requests', true );
		$request              = array();
		$sender_request_key   = '';
		$receiver_request_key = '';
		foreach ( $balance_requests as $key => $balance_request ) {
			if ( $balance_request['id'] == $request_id && $this->user_id == $balance_request['to'] && $balance_request['status'] === 'pending' ) {
				$request            = $balance_request;
				$sender_request_key = $key;
				break;
			}
		}

		if ( empty( $request ) ) {
			return array(
				'status' => false,
				'data'   => _('Empty Request!','wholesalex-pro'),
			);
		}

		$requested_user_id = $balance_request['from'];

		// Verify Request

		$requested_user_balance_requests = get_user_meta( $requested_user_id, '__wholesalex_wallet_balance_requests', true );

		$requested_user_request = array();

		foreach ( $requested_user_balance_requests as $key => $balance_request ) {
			if ( $balance_request['id'] === $request_id && $requested_user_id === $balance_request['from'] && $balance_request['status'] === 'pending' ) {
				$requested_user_request = $balance_request;
				$receiver_request_key   = $key;
				break;
			}
		}

		if ( ! ( $requested_user_request['id'] === $request['id'] ) ) {
			return array(
				'status' => false,
				'data'   => __('Invalid Request!','wholesalex-pro'),
			);

		}

		$sender_account_balance = wholesalex_wallet()->get_wholesalex_balance( $this->user_id, 'edit' );
		$requested_amount       = $request['amount'];
		$request_id             = $request['id'];

		$accepted = false;
		if ( floatval( $sender_account_balance ) >= $requested_amount ) {
			$request_transaction_id = wholesalex_wallet()->initiate_transaction( $requested_user_id, $requested_amount, 'credit', sprintf( __("Credit Request #%s Accepted",'wholesalex-pro'),$request_id ) );
			$this->set_user_id( get_current_user_id() );
			$sender_transaction_id = wholesalex_wallet()->initiate_transaction( $this->user_id, $requested_amount, 'debit', sprintf(__("Credit Request #%s Accepted. Transaction ID %s",'wholesalex-pro'),$request_id,$request_transaction_id) );

			if ( ! $sender_transaction_id && $request_transaction_id ) {
				$status = wholesalex_wallet()->initiate_transaction( $requested_user_id, $requested_amount, 'debit', sprintf( __("Credit Request #%s Reverse",'wholesalex-pro'),$request_id ) );
				$this->set_user_id( get_current_user_id() );
			} else {
				$accepted = true;
			}

			if ( $accepted ) {
				$balance_requests[ $sender_request_key ]['status']                                 = 'accepted';
				$balance_requests[ $sender_request_key ]['transaction_id']                         = $sender_transaction_id;
				$balance_requests[ $sender_request_key ]['transaction_timestamp']                  = date( 'U' );
				$requested_user_balance_requests[ $receiver_request_key ]['status']                = 'accepted';
				$requested_user_balance_requests[ $receiver_request_key ]['transaction_id']        = $receiver_request_key;
				$requested_user_balance_requests[ $receiver_request_key ]['transaction_timestamp'] = date( 'U' );

				update_user_meta( $this->user_id, '__wholesalex_wallet_balance_requests', $balance_requests );
				update_user_meta( $requested_user_id, '__wholesalex_wallet_balance_requests', $requested_user_balance_requests );
				
				do_action('wholesalex_wallet_subaccount_balance_request_approve',$request_id,$requested_user_id , get_current_user_id(), $balance_requests[ $sender_request_key ]);
				
				return array(
					'status' => true,
					'data'   => __('Successfully Completed','wholesalex-pro'),
				);
			}
			return array(
				'status' => false,
				'data'   => __('Error Occured!','wholesalex-pro'),
			);

		} else {
			return array(
				'status' => false,
				'data'   => __('Do not have enough balance to complete this transaction.','wholesalex-pro'),
			);
		}
	}

	/**
	 * Reject Balance Reqeust
	 *
	 * @param string $request_id Request ID.
	 * @return void
	 * @since 1.1.4
	 */
	public function reject_balance_request( $request_id = '' ) {
		$balance_requests     = get_user_meta( get_current_user_id(), '__wholesalex_wallet_balance_requests', true );
		$request              = array();
		$sender_request_key   = '';
		$receiver_request_key = '';
		foreach ( $balance_requests as $key => $balance_request ) {
			if ( $balance_request['id'] == $request_id && $this->user_id == $balance_request['to'] && $balance_request['status'] == 'pending' ) {
				$request            = $balance_request;
				$sender_request_key = $key;
				break;
			}
		}

		if ( empty( $request ) ) {
			return array(
				'status' => false,
				'data'   => __('Empty Request!','wholesalex-pro'),
			);

		}

		$requested_user_id = $balance_request['from'];

		// Verify Request

		$requested_user_balance_requests = get_user_meta( $requested_user_id, '__wholesalex_wallet_balance_requests', true );

		$requested_user_request = array();

		foreach ( $requested_user_balance_requests as $key => $balance_request ) {
			if ( $balance_request['id'] === $request_id && $requested_user_id === $balance_request['from'] ) {
				$requested_user_request = $balance_request;
				$receiver_request_key   = $key;
				break;
			}
		}

		if ( ! ( $requested_user_request['id'] === $request['id'] ) ) {
			return array(
				'status' => false,
				'data'   => __('Invalid Request!','wholesalex-pro'),
			);
		}

		$balance_requests[ $sender_request_key ]['status']             = 'rejected';
		$balance_requests[ $sender_request_key ]['rejected_timestamp'] = date( 'U' );

		$requested_user_balance_requests[ $receiver_request_key ]['status']             = 'rejected';
		$requested_user_balance_requests[ $receiver_request_key ]['rejected_timestamp'] = date( 'U' );

		update_user_meta( $this->user_id, '__wholesalex_wallet_balance_requests', $balance_requests );
		update_user_meta( $requested_user_id, '__wholesalex_wallet_balance_requests', $requested_user_balance_requests );

		do_action('wholesalex_wallet_subaccount_balance_request_reject',$request_id,$requested_user_id , get_current_user_id(), $balance_requests[ $sender_request_key ]);

		return array(
			'status' => true,
			'data'   => __('Successfully Rejected','wholesalex-pro'),
		);
	}

	/**
	 * Is order is a wallet partial payment order
	 *
	 * @param int|string $item_id Item ID.
	 * @param object $item_id Item.
	 * @return boolean
	 * @since 1.1.8
	 */
	public function is_partial_payment_order($item_id, $item) {
		$fee_name = apply_filters('wholesalex_wallet_partial_pay_fee_name',__('WholesaleX Wallet Pay','wholesalex-pro'));
		if ( ( get_metadata( 'order_item', $item_id, '_legacy_fee_key', true ) && '__wholesalex_wallet_partial_payment' === get_metadata( 'order_item', $item_id, '_legacy_fee_key', true ) ) || $fee_name ===  $item->get_name( 'edit' ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Get Partial Payment Amount From An Order
	 *
	 * @param int|string $order_id Order id.
	 * @return float
	 */
	public function get_partial_payment_amount($order_id) {
		$order = wc_get_order($order_id);
		$amount = 0;
		if($order) {
			$fees = $order->get_items( 'fee' );
			foreach ( $fees as $item_id => $item ) {
				$fee_name = apply_filters('wholesalex_wallet_partial_pay_fee_name',wholesalex()->get_language_n_text('_language_wallet_partial_paid_checkout_text','WholesaleX Wallet Pay'));
				if ( ( get_metadata( 'order_item', $item_id, '_legacy_fee_key', true ) && '__wholesalex_wallet_partial_payment' === get_metadata( 'order_item', $item_id, '_legacy_fee_key', true ) ) || $fee_name ===  $item->get_name( 'edit' ) ) {
					$amount += $item->get_total( 'edit' ) + $item->get_total_tax( 'edit' );
				}
			}
		}

		return apply_filters('wholesalex_wallet_get_partial_payment_amount',-1*$amount, $order_id);
	}


	/**
	 * Process Partial Payment
	 *
	 * @param int|string $order_id Order ID.
	 * @return void
	 */
	public function process_partial_payment($order_id) {
		$order = wc_get_order($order_id);
		$amount = floatval($this->get_partial_payment_amount($order_id));
		$is_already_completed = get_post_meta($order_id,'__wholesalex_wallet_partial_payment_transaction_id',true);
		if($amount>0 && !$is_already_completed ) {
			$available_balance = floatval($this->get_wholesalex_balance($order->get_customer_id(),'edit'));
			if($available_balance>=$amount) {
				// procced_transaction
				$reason = apply_filters('wholesalex_wallet_partial_payment_reason',__('For Partial Payment, #','wholesalex-pro'). $order->get_order_number(),$order );
				$transaction_id = $this->initiate_transaction($order->get_customer_id(),$amount,'debit',$reason);
				if($transaction_id) {
					$order_note = apply_filters('wholesalex_wallet_partial_payment_order_note',sprintf(__("Partially Paid %s, Through Wallet",'wholesalex-pro'),wc_price($amount)),$amount,$order);
					$order->add_order_note($order_note);
					update_post_meta($order_id,'__wholesalex_wallet_partial_payment_transaction_id',$transaction_id);

					do_action('wholesalex_wallet_partial_payment_completed',$order,$transaction_id);
				}

			} else {
				do_action('wholesalex_wallet_partial_payment_insufficient_balance',$order,$amount);
			}
		}
	}

	/**
	 * Proces Cancellation of partial order
	 *
	 * @param int|string $order_id Order ID
	 * @return void
	 */
	public function process_partial_payment_cancel_order($order_id) {
		$order = wc_get_order($order_id);
		if($order) {
			$amount = floatval($this->get_partial_payment_amount($order_id));
			if($amount && get_post_meta($order_id,'__wholesalex_wallet_partial_payment_transaction_id',true) ) {
				//refund wallet credit
				// procced_transaction
				$reason = apply_filters('wholesalex_wallet_partial_payment_refund_reason',__('Refund Wallet Balance , #','wholesalex-pro'). $order->get_order_number(),$order );
				$transaction_id = $this->initiate_transaction($order->get_customer_id(),$amount,'credit',$reason);
				if($transaction_id) {
					$order_note = apply_filters('wholesalex_wallet_partial_payment_order_note',sprintf(__("Wallet Balance %s refunded to the customer",'wholesalex-pro'),wc_price($amount)),$amount,$order);
					$order->add_order_note($order_note);
					delete_post_meta($order_id,'__wholesalex_wallet_partial_payment_transaction_id');
					update_post_meta($order_id,'__wholesalex_wallet_partial_payment_refund_transaction_id',$transaction_id);

					do_action('wholesalex_wallet_partial_payment_refunded',$order,$transaction_id);
				}
				
			}

			
		}
	}

}
